/*****************************************************************************
 * Copyright (C) 2008-2014
 *   
 * Michiel Broek <mbroek at mbse dot eu>
 *
 * This file is part of the mbsePi-apps
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * mbsePi-apps is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with EC-65K; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *****************************************************************************/

#include "../config.h"
#include "mbselib.h"


bool		debug = FALSE;
static char	*mypath;
static char	*k, *v;
static int	linecnt = 0;
sys_config	Config;			/* System configuration		*/



static int getstr(char **);
static int getint(char **);
//static int getbyt(char **);
//static int gethex(char **);

#define XSTR(x) #x
#define STR(x) XSTR(x)

/*
 * System configuration table
 */
key_list keytab[] = {
    {(char *)"mosq_host",	getstr,		&Config.mosq_host},
    {(char *)"mosq_port",	getint,		(char **)&Config.mosq_port},
    {NULL,			NULL,		NULL}
};



void killconfig(void)
{
    if (Config.name)
	free(Config.name);
    Config.name = NULL;

    if (Config.mosq_host)
	free(Config.mosq_host);
    Config.mosq_host= (char *)"localhost";
    Config.mosq_port = 1883;
}



int rdconfig(void) 
{
    char	buf[256], *p;
    FILE	*fp;
    int		i, rc = 0;

    killconfig();

    /*
     * Get config from the system path
     */
    mypath = xstrcpy((char *)STR(ETCDIR));
    mypath = xstrcat(mypath, (char *)"/mbsepi-apps.conf");

    if ((fp = fopen(mypath, "r")) == NULL) {
	syslog(LOG_NOTICE, "rdconfig: could not open %s", mypath);
	return 1;
    }

    linecnt = 0;
    while (fgets(buf, sizeof(buf) -1, fp)) {
	linecnt++;
	if (*(p = buf + strlen(buf) -1) != '\n') {
	    syslog(LOG_NOTICE, "rdconfig: %s(%d): \"%s\" - line too long", mypath, linecnt, buf);
	    rc = 1;
	    break;
	}
	*p-- = '\0';
	while ((p >= buf) && isspace(*p))
	    *p-- = '\0';
	k = buf;
	while (*k && isspace(*k))
	    k++;
	p = k;
	while (*p && !isspace(*p))
	    p++;
	*p++='\0';
	v = p;
	while (*v && isspace(*v)) 
	    v++;

	if ((*k == '\0') || (*k == '#')) {
	    continue;
	}

	for (i = 0; keytab[i].key; i++)
	    if (strcasecmp(k,keytab[i].key) == 0)
		break;

	if (keytab[i].key == NULL) {
	    syslog(LOG_NOTICE, "rdconfig: %s(%d): %s %s - unknown keyword", mypath, linecnt, MBSE_SS(k), MBSE_SS(v));
	    rc = 1;
	    break;
	} else if ((keytab[i].prc(keytab[i].dest))) {
	    rc = 1;
	    break;
	}

    }
    fclose(fp);

    free(mypath);
    mypath = NULL;

    return rc;
}



static int getstr(char **dest)
{
    if (debug)
	syslog(LOG_NOTICE, "rdconfig: getstr: %s(%d): %s %s", mypath, linecnt, MBSE_SS(k), MBSE_SS(v));

    *dest = xstrcpy(v);
    return 0;
}



static int getint(char **dest)
{
    if (debug)
	syslog(LOG_NOTICE, "rdconfig: getint: %s(%d): %s %s", mypath, linecnt, MBSE_SS(k), MBSE_SS(v));

    if (strspn(v,"0123456789") != strlen(v)) 
	syslog(LOG_NOTICE, "rdconfig: %s(%d): %s %s - bad numeric", mypath, linecnt, MBSE_SS(k), MBSE_SS(v));
    else 
	*((int*)dest)=atoi(v);
    return 0;
}


/*
static int getbyt(char **dest)
{
    Log_Msg("[rdconfig] getbyt: %s(%d): %s %s", mypath, linecnt, k, v);
    if (strspn(v,"0123456789") != strlen(v))
	Log_Msg("[rdconfig] %s(%d): %s %s - bad numeric", mypath, linecnt, S(k), S(v));
    else
	*((Uint8*)dest)=atoi(v);
    return 0;
}
*/


/*
static int gethex(char **dest)
{
    unsigned int    val = 0;
    int		    rc;

    Log_Msg("[rdconfig] gethex: %s(%d): %s %s", mypath, linecnt, k, v);
    rc = sscanf(v, "%08x", &val);
    if (rc != 1) {
	Log_Msg("[rdconfig] %s(%d): %s %s - bad hex value", mypath, linecnt, S(k), S(v));
	return 1;
    }
    *((int*)dest) = val;

    return 0;
}
*/


